#ifndef HashTable_INCLUDED
#define HashTable_INCLUDED

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : HashTable.h
//
//   Created : 09/23/98
//   By : Patrick McDaniel
//
//   Description : This class provides a generic interface for creating
//                 hash tables of (Listable) objects.
//
//   *** Change History ***
//
//   09/24/98 Created File : DPM
//   10/27/98 Continued performance monitor developement : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***

// Project Include Files
#include <List.h>

// *** Definitions *** 

// Class Definition HashTable
class HashTable : public List {

  public: // Public Interface

    //
    // Constructors and Destructors

    HashTable( int htype, int tablesize = 64 );
      // Default Constructor

    virtual ~HashTable( void );
      // Destructor

    //
    // Member Functions

    virtual int addElement( Listable *ele );
      // Add an element to the list

    virtual Listable *removeElement( Listable *ele );
      // Remove an element from the list.

    virtual Listable *removeElement( void *value );
      // Remove an element from the list (by value)

    virtual Listable *findElement( Listable *ele ); 
      // Find and return an element from list

    virtual Listable *findElement( void *value ); 
      // Find and return an element from list

    virtual int numElements( void );
      // Number of elements in the list

    // Iterator Functions

    virtual void resetIterator( void );
      // Reset the iterator to beginning of list.

    virtual Listable *iterator( void );
      // Interator function over list

    // Debug Functions

    virtual void printList( int lvl, const unsigned char *txt );
      // print to the debug log the contents of the linked list

  private: // Private Interface and Data

    // 
    // Class Attributes

    List **hTable;
      // This is the hash table itself

    int tableSize;
      // This is the size of the table (in lists).

    int iteratorIndex;
      // Index into the hash table iterator

    unsigned int listSize;
      // This is the number of elements in the hash table.

};
// End Class Definition HashTable

#endif
