#ifndef LinkedList_INCLUDED
#define LinkedList_INCLUDED

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : LinkedList.h
//
//   Created : 09/23/98
//   By : Patrick McDaniel
//
//   Description : This class provides a generic interface for creating
//                 linked lists of (Listable) objects.
//
//   *** Change History ***
//
//   09/23/98 Created File : DPM
//   11/03/98 Finished pre-release : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***

// Project Include Files
#include <List.h>

// *** Definitions *** 

// *** Type Definitions *** 

// Typedef used for creating lists
typedef struct listable_struct {
  struct listable_struct *next;  // Next object in list
  Listable               *value; // Value associated with entry
} listable;

// Class Definition LinkedList
class LinkedList : public List {

  public: // Public Interface

    //
    // Constructors and Destructors

    LinkedList( int htype, int sorted = true );
      // Default Constructor

    virtual ~LinkedList( void );
      // Destructor

    //
    // Member Functions

    virtual int addElement( Listable *ele );
      // Add an element to the list

    virtual Listable *removeElement( Listable *ele );
      // Remove an element from the list.

    virtual Listable *removeElement( void *value );
      // Remove an element from the list (by value)

    virtual Listable *findElement( Listable *ele ); 
      // Find and return an element from list

    virtual Listable *findElement( void *value ); 
      // Find and return an element from list

    virtual int numElements( void );
      // Number of elements in the list

    // Iterator Functions

    virtual void resetIterator( void );
      // Reset the iterator to beginning of list.

    virtual Listable *iterator( void );
      // Interator function over list

    //
    // Debug Functions

    virtual void printList( int lvl, const unsigned char *txt );
      // print to the debug log the contents of the linked list

    //
    // Member Functions

    int randomReorder( void );
      // randomly order the list elements

  private: // Private Interface and Data

    // 
    // Class Attributes

    listable *head, *tail, *interatorPtr;
      // This is the structure that points to the items in list.

    unsigned int listSize;
      // This is the number of elements in the list.

};
// End Class Definition LinkedList

#endif
