#ifndef List_INCLUDED
#define List_INCLUDED

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : List.h
//
//   Created : 09/23/98
//   By : Patrick McDaniel
//
//   Description : This class provides a generic interface for creating
//                 lists of (Listable) objects.
//
//   *** Change History ***
//
//   09/23/98 Created File : DPM
//   10/07/98 Added member function atIndex() : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***

// Project Include Files
#include <Listable.h>

// *** Definitions *** 

#if !defined(true)
#define true 1
#endif
#if !defined(false)
#define false 0
#endif


// Hash Value Types
#define STRING_HASH 0x1
#define UL_HASH     0x2

// Class Definition List
class List {

  public: // Public Interface

    //
    // Constructors and Destructors

    // Default Constructor
    List( int htype, int sorted ) 
      : hashType( htype ),
        isSorted( sorted )
    {
        // No return value
    }

    //
    // Access Functions

    // Return the type of hash used
    int HashType( void )
    {
        return( hashType );
    }

    // Return sorted flag
    int Sorted( void )
    {
        return( isSorted );
    }

    //
    // Virtual Functions

    virtual int addElement( Listable *ele ) = 0;
      // Add an element to the list

    virtual Listable *removeElement( Listable *ele ) = 0;
      // Remove an element from the list.

    virtual Listable *removeElement( void *value ) = 0;
      // Remove an element from the list (by value)

    virtual Listable *findElement( Listable *ele ) = 0; 
      // Find and return an element from list

    virtual Listable *findElement( void *value ) = 0; 
      // Find and return an element from list

    virtual int numElements( void ) = 0;
      // Number of elements in the list

    // Iterator Functions

    virtual void resetIterator( void ) = 0;
      // Reset the iterator to beginning of list.

    virtual Listable *iterator( void ) = 0;
      // Interator function over list

    // Debug Functions

    virtual void printList( int lvl, const unsigned char *txt ) = 0;
      // print to the debug log the contents of the linked list

    //
    // Member functions

    // Return true if listable in list
    int In( Listable *ele )
    {
        return( findElement(ele) != NULL );
    }

    // Return true if listable in list
    int In( void *value )
    {
        return( findElement(value) != NULL );
    }

    Listable *atIndex( int idx );
      // This is a generic indexing function.  It is very ineffecient
      // and should be overridden.  (0 based indexing)

    char * ListString( void );
      // Convert list into string

    unsigned int hashValue( void *value );
      // Convert string into hash value

    List *addList( List *list );
      // Add all elements from one list to another.

    void cleanList( void );
      // Empty the contents of the list.

  protected: // Private Interface and Data

    // 
    // Class Attributes

    int hashType;
      // This is the type of hashing value used.

    int isSorted;
      // Is this list sort or not flag.

};
// End Class Definition List

#endif
