#ifndef Metric_INCLUDED
#define Metric_INCLUDED

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : Metric.h
//
//   Created : 10/27/98
//   By : Patrick McDaniel
//
//   Description : This class describes one instance of a generic
//                 performance metric.
//
//   *** Change History ***
//
//   10/27/98 Created File : DPM
//   10/27/98 Continued performance monitor developement : DPM
//   1/29/99  changed mark to accept both seconds and usec
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***

// Project Include Files
#include <Listable.h>
#include <Monitor.h>
#include <MetricFilter.h>

// *** Definitions *** 

// Metric Types
#define COUNT_METRIC_TYPE  0xf0
#define MAX_METRIC_TYPE    0xf1
#define AVG_METRIC_TYPE    0xf2

// Class Definition Metric
class Metric : public Listable {

  public: // Public Interface

    //
    // Constructors and Destructors

    Metric( int indx );
      // Attribute constructor

    virtual ~Metric( void );
      // Destructor

    //
    // Access Functions

    // Return the identifier of this metric
    int ID( void )
    {
        return( metricId );
    }

    //
    // Derived Class Methods

    virtual unsigned int hashValue( void );
      // Hash value of object.

    virtual void *listValue( void );
      // Get the ordering list value (not to be interpreted)

    virtual int compareListValues( void *value );
      // Make a comparison between the two values.

    // This returns the type of object (listable def)
    virtual int typeOf( void )
    {
        return( TYPE_METRIC );
    }

    //
    // Abstract Class Functions

    //virtual void metricEvent( unsigned long val ) = 0;
    virtual void metricEvent( double val ) = 0;
      // update the metric with a given value

    //virtual void mark( unsigned long tm ) = 0;
    virtual void mark(unsigned long sec, unsigned long usec) = 0; 
      // trigger the update of the filters based on some time
      // reference

    virtual void markSummary( unsigned long tm ) = 0;
      // mark the summary statistics of the metric

    // 
    // Member Functions

    void registerFilter( MetricFilter *fil );
      // Register the filter to be contacted during mark phase.

    void unregisterFilter( MetricFilter *fil );
      // Unregister a filter that was contacted during mark phase.

    //
    // Static Member Functions

    static Metric * MetricFactory( int id, int typ );
      // Create a matric from the factory.

  protected: // Derived class accessable 

    List *filters;
      // This is the list of metric filters that are contacted during
      // the mark phase.

  private: // Private Interface and Data

    // 
    // Class Attributes

    int metricId;
      // This ID uniquely identifies the metric.

};
// End Class Definition Metric

#endif
