#ifndef MetricFilter_INCLUDED
#define MetricFilter_INCLUDED

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : MetricFilter.h
//
//   Created : 10/27/98
//   By : Patrick McDaniel
//
//   Description : This (abstract) class describes one instance of a
//                 generic performance metric filter.  The purpose of
//                 this class is to organisze and output the data into
//                 a meaningful output.
//
//   *** Change History ***
//
//   10/27/98 Created File : DPM
//   10/27/98 Continued performance monitor developement : DPM
//   11/03/98 Completed version 1.0 : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***

// Project Include Files
#include <Listable.h>
#include <Monitor.h>

// *** Definitions *** 
#define METRIC_FIELD_WIDTH 8

// Class Definition MetricFilter
class MetricFilter : public Listable {

  public: // Public Interface

    //
    // Constructors and Destructors

    MetricFilter( void );
      // Default constructor

    virtual ~MetricFilter( void );
      // Destructor

    //
    // Access Functions

    // Return the identifier of this metric
    int ID( void )
    {
        return( filterId );
    }

    //
    // Derived Class Methods

    virtual unsigned int hashValue( void );
      // Hash value of object.

    virtual void *listValue( void );
      // Get the ordering list value (not to be interpreted)

    virtual int compareListValues( void *value );
      // Make a comparison between the two values.

    virtual unsigned char *toString( void );
      // Construct a string used for printing.

    // This returns the type of object (listable def)
    virtual int typeOf( void )
    {
        return( TYPE_METRIC_FILTER );
    }

    // 
    // Member Functions

    virtual void mark( int metID, unsigned long sec,unsigned long usec, double val ) = 0;
      // update ocurrent filter state with new values

    virtual void markSummary( int metID, unsigned long tm, double val ) = 0;
      // mark the summary statistics of the metric

    //
    // Static Member Functions

  private: // Private Interface and Data

    // 
    // Class Attributes

    int filterId;
      // This ID uniquely identifies the filter

    //
    // Static Class Attributes

    static int nextFilterId;
        // This is counter used to assign filter ids.

};
// End Class Definition MetricFilter

#endif
