#ifndef Utils_INCLUDED
#define Utils_INCLUDED

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : Utils.h
//
//   Created : 09/22/98
//   By : Patrick McDaniel
//
//   Description : This static class contains general purpose functions
//                 used within the KDH simulator tools.
//
//   *** Change History ***
//
//   09/22/98 Created File : DPM
//   09/30/98 Added parameter functions : DPM
//   11/03/98 Finished pre-release : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***

// Project Include Files

// *** Definitions *** 

// Class Definition Utils
class Utils {

  public: // Exception Definition

      //
      // Exception Thrown when input data is bad

      class DataFormatException {

       public : // Exception Interface
  
            // Constructor
            DataFormatException( char *nm ) : name(nm) {}
  
            // Access Function
            char * getName( void ) { return( name ); }
  
        private : // Exception Data
  
            char *name;
              // This is the parameter name reading when exception thrown.
  
      }; // End of Exception Definition

  public: // Public Interface

    // 
    // Static Member Functions

    static unsigned char *getToken( unsigned char *line, unsigned char *buf, 
                                    int buflen );
      // get a token from the input line, placing the token in the
      // input buffer.

    static unsigned char *toLower( unsigned char *line );
      // convert a string to lower case

    static int hashValue( unsigned char *str );
      // compute a hash value for a string

    static int readParameter( unsigned char *line, char *name );
      // has no value, just check existance

    static int readParameter( unsigned char *line, char *name, int *value );
    static int readParameter( unsigned char *line, char *name, 
                              unsigned long *value );
    static int readParameter( unsigned char *line, char *name, double *value );
    static int readParameter( unsigned char *line, char *name, 
                              unsigned char **value );
      // Read the parameter from the input line (depositing the 
      // value in the value reference) 

    static int readParameter( unsigned char *line, char *name, 
                              double *val1, double *val2 );
      // Read multiparameter on line

    static int readParameter( unsigned char *line, char *name, 
                              unsigned long **value );
      // Read the array from the input line (depositing the value in
      // the value reference)

};
// End Class Definition Utils

#endif
