/*

    File : dbgLog.h

    Description : This file contains definitions for support routines that 
                  define a flexible interface for creating quickly modifiable 
                  debugging levels.

    Created : 9/15/98
    By      : Patrick McDaniel

    Change History
    ----
    09/15/98 PDM File Created

*/

/* Include Files */
#include <string.h>
#ifdef AIX
#if !defined(true)
#define true 1
#endif
#include <stdlib.h>
#endif

/* Macro Definitions */
#define SET_LOG_LEVEL(x) (_dbgLogLevel=x)
#define ADD_LOG_LEVEL(x) (_dbgLogLevel|=x)
#define DEL_LOG_LEVEL(x) (_dbgLogLevel=((~x)&_dbgLogLevel))
#define LOG_LEVEL_ON(x)  (_dbgLogLevel&x)

#define SET_TERM_ECHO(x) (echoTerm=x)
#define SET_LOG_ECHO(x) (echoLog=x)
#define SET_LOG_FILENAME(x) (logFilename=(unsigned char *)strdup(x))

#define DBG_LOG_MSG_FLUSH 5
#define DEFAULT_LOG_FILENAME ((unsigned char *)"dbgLog.log")

/* Dbg Log Levels 

  32 bit mask

   0-15 - system debug levels
  16-31 - application debug levels

*/

/* System Debug Levels */
#define DBG_INFORM       0x00000001L
#define DBG_FATAL_ERROR  0x00000002L
#define DBG_MAJOR_ERROR  0x00000004L
#define DBG_MINOR_ERROR  0x00000008L
#define DBG_VERIFICATION 0x00000010L
#define DBG_MONITOR      0x00000020L

/* Application Specific Debug Levels */
#define DBG_TRAFFIC_GEN        0x00010000L
#define DBG_SIMULATE           0x00020000L
#define DBG_READ_DEFINITION    0x00040000L
#define DBG_TRACE_CACHE        0x00080000L
#define DBG_REQUEST_PROCESSING 0x00100000L
#define DBG_CRL_CACHE          0x00200000L
#define DBG_CRL_PROCESSING     0x00400000L

/* NOTE: When adding new debug levels, be sure to add to text array */

/* Default Log Level (should only be modified through macros) */
#define DEFAULT_DBG_LEVEL ( DBG_INFORM | \
                            DBG_FATAL_ERROR | \
                            DBG_MAJOR_ERROR | \
                            DBG_MINOR_ERROR )

/* Debug Log Data */
extern unsigned long _dbgLogLevel;
extern unsigned short echoTerm;
extern unsigned short echoLog;
extern unsigned char *logFilename;

/* Function Definitions */

int _dbgLog( int dbgLevel, char *logEntry );
  /* print a message to the log */

int _dbgLogMsg( int lLevel, char *frmt, ... );
  /* Write a message to the log (with formatting). */

void closeDbgLog( void );
  /* Close the log file. */

const char *fromByteString( const char *str, int len );
  /* create string of bytes suitable for printing. */

unsigned long textLogLevel( char *str );
  /* convert the string of into a log level (exit on any unknown debug
     levels) */

