/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : AverageMetric.cpp
//
//   Created : 11/16/98
//   By : Patrick McDaniel
//
//   Description : (See AverageMetric.h for class description)
//
//   *** Change History ***
//
//   11/16/98 Created File : DPM
//   4/2/99   mark data changed to float  : andros@citi.umich.edu
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <stdio.h>

// Project Include Files
#include <Support.h>
#include <AverageMetric.h>

// Definitions

//
// Constructors and Destructors

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : AverageMetric::AverageMetric
// Description  : Attribute constructor
//
// Inputs       : indx - unique ID for metric
// Outputs      : none

AverageMetric::AverageMetric( int indx )
  : Metric( indx ),
    markCount( 0 ), 
    markTotal( 0 ),
    summaryCount( 0 ), 
    summaryTotal( 0 )
{
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : AverageMetric::~AverageMetric
// Description  : Destructor
//
// Inputs       : none
// Outputs      : none

AverageMetric::~AverageMetric( void )
{
    // Return, no return code
    return;
}

//
// Derived Class Methods

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : AverageMetric::toString
// Description  : Construct a string used for printing.
//
// Inputs       : none
// Outputs      : (static) printable string

unsigned char *AverageMetric::toString( void )
{
    static unsigned char buf[256];
    sprintf( (char *)buf, "AverageMetric id=%02d", ID() );
    return( buf );
}

// 
// Member Functions

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : AverageMetric::AverageMetricEvent
// Description  : update the metric with a given value
//
// Inputs       : val - update value
// Outputs      : none

//void AverageMetric::metricEvent( unsigned long val )
void AverageMetric::metricEvent( double val )
{
    // Update the appropriate metric values
    markCount ++;
    markTotal += val;
    summaryCount ++;
    summaryTotal += val;

    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : AverageMetric::Mark
// Description  : trigger the update of the filters based on some time
//                reference
//
// Inputs       : tm - mark time
// Outputs      : none

void AverageMetric::mark( unsigned long sec, unsigned long usec)
{
     // Trigger update of all filters
    SUPASSERT( (filters != NULL), "Attemp to mark metric with NULL filter" );

    //filters will have no elements if unregistered.
    if(filters->numElements() > 0){

    // Calculate the average
        double average = (((double)markTotal/(double)markCount)+0.5);
    // Trigger update of all filters
        MetricFilter *filter;
        filters->resetIterator();
        while ( (filter = (MetricFilter *)filters->iterator()) != NULL )
            filter->mark( ID(), sec,usec, average );

    // Reset short term metric
        markCount = markTotal = 0;
    }
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : AverageMetric::MarkSummary
// Description  : mark the summary statistics of the metric
//
// Inputs       : tm - time of last occurrence
// Outputs      : none

void AverageMetric::markSummary( unsigned long tm )
{
    // Calculate the average
    unsigned long average =
                  (unsigned long)(((double)summaryTotal/(double)summaryCount)+0.5);

    // Trigger update of all filters
    SUPASSERT( ((filters != NULL) && (filters->numElements() > 0)), 
               "Summ Marking metric with no filters" );
    MetricFilter *filter;
    filters->resetIterator();
    while ( (filter = (MetricFilter *)filters->iterator()) != NULL )
        filter->markSummary( ID(), tm, average );
    
    // Return, no return code
    return;
}

//
// Static Member Functions

