/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : CountMetric.cpp
//
//   Created : 10/27/98
//   By : Patrick McDaniel
//
//   Description : (See CountMetric.h for class description)
//
//   *** Change History ***
//
//   10/27/98 Created File : DPM
//   10/27/98 Continued performance monitor developement : DPM
//   4/2/99   mark data changed to float  : andros@citi.umich.edu
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <stdio.h>

// Project Include Files
#include <Support.h>
#include <CountMetric.h>

// Definitions

//
// Constructors and Destructors

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : CountMetric::CountMetric
// Description  : Attribute constructor
//
// Inputs       : indx - unique ID for metric
// Outputs      : none

CountMetric::CountMetric( int indx )
  : Metric( indx ),
    markValue( 0 ),
    summaryValue( 0 )
{
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : CountMetric::~CountMetric
// Description  : Destructor
//
// Inputs       : none
// Outputs      : none

CountMetric::~CountMetric( void )
{
    // Return, no return code
    return;
}

//
// Derived Class Methods

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : CountMetric::toString
// Description  : Construct a string used for printing.
//
// Inputs       : none
// Outputs      : (static) printable string

unsigned char *CountMetric::toString( void )
{
    static unsigned char buf[256];
    sprintf( (char *)buf, "CountMetric id=%02d", ID() );
    return( buf );
}

// 
// Member Functions

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : CountMetric::CountMetricEvent
// Description  : update the metric with a given value
//		markValue is the short term value reset to zero at mark. 
//
// Inputs       : val - update value
// Outputs      : none
// 

//void CountMetric::metricEvent( unsigned long val )
void CountMetric::metricEvent( double val )
{
    // Update the appropriate metric values
    markValue += val;
    summaryValue += val;

    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : CountMetric::Mark
// Description  : trigger the update of the filters based on some time
//                reference
//
// Inputs       : tm - mark time
// Outputs      : none

void CountMetric::mark( unsigned long sec,unsigned long usec)
{
    // Trigger update of all filters

    SUPASSERT( (filters != NULL), "Attemp to mark metric with NULL filter" );

    //filters will have no elements if unregistered.
    if(filters->numElements() > 0){
        MetricFilter *filter;
        filters->resetIterator();
        while ( (filter = (MetricFilter *)filters->iterator()) != NULL )
            filter->mark( ID(), sec,usec, markValue );

    // Reset short term metric
        markValue = 0;
    }
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : CountMetric::MarkSummary
// Description  : mark the summary statistics of the metric
//
// Inputs       : tm - time of last occurrence
// Outputs      : none

void CountMetric::markSummary( unsigned long tm )
{
    // Trigger update of all filters
    SUPASSERT( ((filters != NULL) && (filters->numElements() > 0)), 
               "Summ Marking metric with no filters" );
    MetricFilter *filter;
    filters->resetIterator();
    while ( (filter = (MetricFilter *)filters->iterator()) != NULL )
        filter->markSummary( ID(), tm, summaryValue );
    
    // Return, no return code
    return;
}

//
// Static Member Functions

