/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : GnuPlotFileFilter.cpp
//
//   Created : 1/29/99
//   By : andros
//
//   Description : (See GnuPlotFileFilter.h for class description)
//
//   *** Change History ***
//   4/2/99   mark data changed to float  : andros@citi.umich.edu
//
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <stdlib.h>
#include <string.h>

// Project Include Files
#include <GnuPlotFileFilter.h>
#include <Support.h>

// Definitions

//
// Static Attributes

//
// Constructors and Destructors

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : GnuPlotFileFilter::GnuPlotFileFilter
// Description  : Attribute constructor
//
// Inputs       : none
// Outputs      : none

GnuPlotFileFilter::GnuPlotFileFilter( char *nm )
  : FileName( strdup(nm) ),
    file_( NULL )
{
    // Create the file
    file_ = fopen(FileName,"w");
    SUPASSERT((file_ != NULL),"Constructing GnuPlotFileFilter");

    mrkarray_index_ = 0;
    int i;
    for(i=0;i<MRKARRAYSIZE;i++){
	mrkarray_[0][i] = 0.0;
	mrkarray_[1][i] = 0.0;
    }

    gettimeofday(&start_,(void *)NULL);
    then_.tv_sec = start_.tv_sec;
    then_.tv_usec = start_.tv_usec;

    //Return, no return code
    return; 
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : GnuPlotFileFilter::~GnuPlotFileFilter
// Description  : Destructor
//
// Inputs       : none
// Outputs      : dumps data array to file.

GnuPlotFileFilter::~GnuPlotFileFilter( void )
{
    int i;
    char markstr[64];
    for(i=0;i<MRKARRAYSIZE;i++){
	if(mrkarray_[0][i] > 0){ 
            memset(markstr,0,64);
            sprintf(markstr,"%.3f %.6f\n",mrkarray_[0][i],mrkarray_[1][i]);
            fputs(markstr,file_);
	}
    }
    fflush(file_);
    mrkarray_index_=0;
    // Cleanup the file
    fclose(file_);

    // Return, no return code
    return;
}

//
// Derived Class Methods

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : GnuPlotFileFilter::Mark
// Description  : update ocurrent filter state with new values
//
// Inputs       : metId - metric ID
//                sec,usec- time of mark
//                val - summary value
// Outputs      : none

void GnuPlotFileFilter::mark( int ID, unsigned long sec,unsigned long usec, double val )
{
char markstr[64];
#if defined(SEC_AND_USEC)
float tempsec;
float temp;
float tmpusec;
float time;


#endif /*SEC_AND_USEC*/
long tempusec;

    memset(markstr,0,64);

    //calculate time from start_
#if defined(SEC_AND_USEC)
    // float format - i.e. 2 sec and 478000 usec becomes 2.478 seconds....

    tempsec = sec - start_.tv_sec;
    tempusec = usec - start_.tv_usec;
    temp = tempusec;
    if(tempusec < 0){
	--tempsec;
	tmpusec = 1000000 + temp;
        time = tempsec + (tmpusec/1000000.0);
    }else
        time = tempsec + (temp/1000000.0);

    if(mrkarray_index_ < MRKARRAYSIZE){
	mrkarray_[0][mrkarray_index_] = time;
	mrkarray_[1][mrkarray_index_] = val;
	mrkarray_index_++;
    }
    else{
	int i;
	for(i=0;i<MRKARRAYSIZE;i++){
	    memset(markstr,0,64);
            sprintf(markstr,"%.3f %.6f\n",mrkarray_[0][i],mrkarray_[1][i]);
            fputs(markstr,file_);
	    mrkarray_[0][i] = 0;
	    mrkarray_[1][i] = 0;
	}
        fflush(file_);
	mrkarray_index_=0;
    }

#else /*SEC_AND_USEC*/
    // long format - convert seconds into usec
   // OLD method - no array.
    tmpval.tv_sec = sec;
    tmpval.tv_usec = usec;
    tempusec = USEC(&start_,&tmpval);
    sprintf(markstr,"%d %.6f\n",tempusec,val);
    fputs(markstr,file_);
    fflush(file_);

#endif /*SEC_AND_USEC*/


    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : GnuPlotFileFilter::MarkSummary
// Description  : mark the summary statistics of the metric (do nothing)
//
// Inputs       : metId - metric ID
//                tm - last measured moment
// Outputs      : none

void GnuPlotFileFilter::markSummary( int, unsigned long, double)
{
    // Return, no return code
    return;
}
