/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : MaxMetric.cpp
//
//   Created : 11/16/98
//   By : Patrick McDaniel
//
//   Description : (See MaxMetric.h for class description)
//
//   *** Change History ***
//
//   11/16/98 Created File : DPM
//   4/2/99   mark data changed to float  : andros@citi.umich.edu
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <stdio.h>

// Project Include Files
#include <dbgLog.h>
#include <Support.h>
#include <MaxMetric.h>

// Definitions

//
// Constructors and Destructors

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MaxMetric::MaxMetric
// Description  : Attribute constructor
//
// Inputs       : indx - unique ID for metric
// Outputs      : none

MaxMetric::MaxMetric( int indx )
  : Metric( indx ),
    maxValue( 0 ),
    summaryMaxValue( 0 )
{
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MaxMetric::~MaxMetric
// Description  : Destructor
//
// Inputs       : none
// Outputs      : none

MaxMetric::~MaxMetric( void )
{
    // Return, no return code
    return;
}

//
// Derived Class Methods

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MaxMetric::toString
// Description  : Construct a string used for printing.
//
// Inputs       : none
// Outputs      : (static) printable string

unsigned char *MaxMetric::toString( void )
{
    static unsigned char buf[256];
    sprintf( (char *)buf, "MaxMetric id=%02d", ID() );
    return( buf );
}

// 
// Member Functions

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MaxMetric::MaxMetricEvent
// Description  : update the metric with a given value
//
// Inputs       : val - update value
// Outputs      : none

void MaxMetric::metricEvent(double val )
{
    // Update the appropriate metric values
    if ( val > maxValue ) 
        maxValue = val;
    if ( val > summaryMaxValue ) 
        summaryMaxValue = val;

    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MaxMetric::Mark
// Description  : trigger the update of the filters based on some time
//                reference
//
// Inputs       : tm - mark time
// Outputs      : none

//void MaxMetric::mark( unsigned long tm )
void MaxMetric::mark( unsigned long sec, unsigned long usec)
{
    // Trigger update of all filters
    SUPASSERT( (filters != NULL), "Attemp to mark metric with NULL filter" );

    //filters will have no elements if unregistered.
    if(filters->numElements() > 0){

        MetricFilter *filter;
        filters->resetIterator();
        while ( (filter = (MetricFilter *)filters->iterator()) != NULL )
            filter->mark( ID(),sec,usec,maxValue );

    // Reset short term metric
        maxValue = 0;
    }
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MaxMetric::MarkSummary
// Description  : mark the summary statistics of the metric
//
// Inputs       : tm - time of last occurrence
// Outputs      : none

void MaxMetric::markSummary( unsigned long tm )
{
    // Trigger update of all filters
    SUPASSERT( ((filters != NULL) && (filters->numElements() > 0)), 
               "Summ Marking metric with no filters" );
    MetricFilter *filter;
    filters->resetIterator();
    while ( (filter = (MetricFilter *)filters->iterator()) != NULL )
        filter->markSummary( ID(), tm, summaryMaxValue );
    
    // Return, no return code
    return;
}

//
// Static Member Functions

