/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : MetricFilter.cpp
//
//   Created : 10/27/98
//   By : Patrick McDaniel
//
//   Description : (See MetricFilter.h for class description)
//
//   *** Change History ***
//
//   10/27/98 Created File : DPM
//   10/27/98 Continued performance monitor developement : DPM
//   4/2/99   mark data changed to float  : andros@citi.umich.edu
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <stdio.h>

// Project Include Files
#include <MetricFilter.h>

// Definitions

//
// Static Class Attributes
int MetricFilter::nextFilterId = 1;

//
// Constructors and Destructors

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MetricFilter::MetricFilter
// Description  : Default constructor
//
// Inputs       : none
// Outputs      : none

MetricFilter::MetricFilter( void )
  : filterId( nextFilterId++ )
{
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MetricFilter::~MetricFilter
// Description  : Destructor
//
// Inputs       : none
// Outputs      : none

MetricFilter::~MetricFilter( void )
{
    // Return, no return code
    return;
}

//
// Derived Class Methods

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MetricFilter::hashValue
// Description  : Hash value of object.
//
// Inputs       : none
// Outputs      : hash value

unsigned int MetricFilter::hashValue( void )
{
    return( filterId );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MetricFilter::listValue
// Description  : Get the ordering list value (not to be interpreted)
//
// Inputs       : none
// Outputs      : pointer to value 

void * MetricFilter::listValue( void )
{
    return( (void *)&filterId );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MetricFilter::compareListValues
// Description  : Make a comparison between the two values.
//
// Inputs       : value - value to compare it to
// Outputs      : < 0 if local value is less than value parameter
//                0 if local value is equal to value parameter
//                > 0 if local value is greater than value parameter

int MetricFilter::compareListValues( void *value )
{
    return( filterId - *((int *)value) );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : MetricFilter::toString
// Description  : Construct a string used for printing.
//
// Inputs       : none
// Outputs      : (static) printable string

unsigned char *MetricFilter::toString( void )
{
    static unsigned char buf[256];
    sprintf( (char *)buf, "MetricFilter id=%02d", filterId );
    return( buf );
}

// 
// Member Functions

//
// Static Member Functions

