/*
 * Copyright 1999
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : Monitor.cpp
//
//   Created : 10/27/98
//   By : Patrick McDaniel
//
//   Description : (See Monitor.h for class description)
//
//   *** Change History ***
//
//   10/27/98 Created File : DPM
//   10/27/98 Continued performance monitor developement : DPM
//   4/2/99   mark data changed to float  : andros@citi.umich.edu
//
//////////////////////////////////////////////////////////////////////////////////

// Include Files
#include <stdio.h>

// Project Include Files
#include <dbgLog.h>
#include <Monitor.h>
#include <Metric.h>
#include <sys/time.h>

// Definitions

//
// Static Attributes

//
// Constructors and Desttructors

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Monitor::Monitor
// Description  : Default constructor
//
// Inputs       : none
// Outputs      : none

Monitor::Monitor( void )
  : metrics( new HashTable(UL_HASH,32) )
{
    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Monitor::~Monitor
// Description  : Destructor
//
// Inputs       : none
// Outputs      : none

Monitor::~Monitor( void )
{
    delete metrics;

    // Return, no return code
    return;
}

// 
// Member Functions

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Monitor::RegisterMetric
// Description  : Register a new collection metric with the monitor.
//
// Inputs       : metIdx - unique identifier for this metric
//                typ - metric type identifier
// Outputs      : return pointer to new metric or NULL if error

Metric* Monitor::registerMetric( int metIdx, int typ )
{
    // If not currently defined, add it
    if ( metrics->findElement( &metIdx ) != NULL )
    {
        // Debug bad registration
        _dbgLogMsg( DBG_MINOR_ERROR, "Bad metric registation, already exists (%d)",
                    metIdx );
        return( NULL );
    }

    // Create the metric
    Metric *metric = Metric::MetricFactory( metIdx, typ );

    // Debug registration and register
    _dbgLogMsg( DBG_MONITOR, "Metric registation METRIC [%s]", 
                metric->toString() );
    metrics->addElement( metric );

    // Return, no return code
    return( metric );
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Monitor::Mark
// Description  : trigger the update of the filters based on some time
//                reference
//
// Inputs       :// tm - mark time
//		: sec  - mark time
//		: usec - mark time
// Outputs      : none

//void Monitor::mark( unsigned long tm )
void Monitor::mark( unsigned long sec, unsigned long usec)
{
    // Iterate through the mertics, marking each
    Metric *metric;
    metrics->resetIterator();
    while ( (metric = (Metric *)metrics->iterator()) != NULL )
        metric->mark( sec,usec );

    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Monitor::MarkSummary
// Description  : mark the summary statistics of the metric
//
// Inputs       : tm - time at which summary indicated
// Outputs      : none

void Monitor::markSummary( unsigned long tm )
{
    // Iterate through the mertics, marking each
    Metric *metric;
    metrics->resetIterator();
    while ( (metric = (Metric *)metrics->iterator()) != NULL )
    {
        //metric->mark(tm);
	//ANDROS: added usec to mark call. don't know if i 
	//want to keep it, so don't change marksummary...
        metric->mark(tm,0);
        metric->markSummary(tm);
    }

    // Return, no return code
    return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Monitor::MetricEvent
// Description  : Register an event with monitor.
//
// Inputs       : idx - unique identifier for this metric
//                val - event value
// Outputs      : none

void Monitor::metricEvent( int idx, double val )
{
    // Get the metric, and update it
    Metric *metric = (Metric *)metrics->findElement( &idx );
    if ( metric == NULL )
        // Debug the error
        _dbgLogMsg( DBG_MAJOR_ERROR, "Unable to find metric to update [%d]", idx );
    else
        metric->metricEvent( val );

    // Return, no return code
    return;
}
//////////////////////////////////////////////////////////////////////////////////
//
// Function     : Monitor::FindMetric
// Description  : Return the desired Metric
//
// Inputs       : idx - unique identifier for this metric
// Outputs      : metric pointer

Metric *Monitor::FindMetric( int idx )
{
    // Get the metric, and update it
    Metric *metric = (Metric *)metrics->findElement( &idx );
    if ( metric == NULL ){
        // Debug the error
        _dbgLogMsg(DBG_MAJOR_ERROR,"FindMetric:Unable to find metric [%d]",idx);
	return (Metric *)NULL;
    }
    else
	return metric;

}
