/*
 * Copyright 1995
 * Center for Information Technology Integration (CITI)
 * Information Technology Division
 * The University of Michigan
 * Ann Arbor
 * info@citi.umich.edu
 *
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies of the software and modified versions or derivative works
 * thereof and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the names of
 * CITI or THE UNIVERSITY OF MICHIGAN not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." CITI and THE UNIVERSITY OF MICHIGAN
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
 * CITI OR THE UNIVERSITY OF MICHIGAN BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Created by: andros@citi.umich.edu 4/2/99
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <strings.h>
#include <signal.h>

#include <Metric.h>
#include <GnuPlotFileFilter.h>
#include <qos_monitor_id.h>

GnuPlotFileFilter *wFilter_bw = new GnuPlotFileFilter( "recv_gen_bw" );
GnuPlotFileFilter *wFilter_pl = new GnuPlotFileFilter( "recv_gen_pl" );

void
exit_handler(int i)
{
    wFilter_bw->~GnuPlotFileFilter();
    wFilter_pl->~GnuPlotFileFilter();
    exit(0);
}


int debug = 0; /* toggle debug printf */
main()
{
int fd, len, n;
struct sockaddr_in saddr;
char buf[2048];
long mark_interval = 0;
long seq_no,s_no,prev_no;
long mark_no = 0;

    if ((fd = socket(PF_INET, SOCK_DGRAM, 0)) < 0) {
        perror("socket");
	exit(1);
    }

    signal(SIGINT,exit_handler);

    saddr.sin_family = AF_INET;
    saddr.sin_port = 0;
    saddr.sin_addr.s_addr = INADDR_ANY;

    if (bind(fd,(struct sockaddr *)&saddr, sizeof saddr) < 0) {
	perror("bind");
	exit(1);
    }
    bzero(&saddr, sizeof saddr);
    len = sizeof saddr;
    if (getsockname(fd,(struct sockaddr *)&saddr, &len) < 0) {
	perror("getsockname");
	exit(1);
    }

    /* tell gen_send the port and address to send to */

   struct timeval start;
   gettimeofday(&start,(void *)NULL);

    printf("port %d\n", ntohs(saddr.sin_port));
    fflush(stdout);

    Monitor *mon = new Monitor();

    Metric *metric_bw = mon->registerMetric( RECV_BANDWIDTH_METRIC_ID, COUNT_METRIC_TYPE );
    metric_bw->registerFilter( wFilter_bw );
    Metric *metric_pl = mon->registerMetric( PACKET_LOSS_METRIC_ID, COUNT_METRIC_TYPE );
    metric_pl->registerFilter( wFilter_pl );

    /* beginning of mark time interval */
    struct timeval then;
    gettimeofday(&then,NULL);

    prev_no = 0;
    len = sizeof saddr;
    for(;;){
        if ((n = recvfrom(fd, buf, sizeof buf, 0, (struct sockaddr *)&saddr, &len)) < 0) {
            perror("recvfrom");
            exit(1);
        }
	(void)bcopy(buf,&s_no,sizeof(seq_no));
	seq_no = ntohl(s_no);
	if(debug)printf("SEQ: %d  #BITS %d LOST %d\n",seq_no,n*8,seq_no - prev_no-1 );
	if((seq_no != 0) && (seq_no != prev_no + 1)){
	    mon->metricEvent(PACKET_LOSS_METRIC_ID,seq_no - prev_no - 1);
        }
	prev_no = seq_no;
	/* record packet loss */
	/* record number of bits received */
        mon->metricEvent( RECV_BANDWIDTH_METRIC_ID, n * 8 );

        struct timeval now;
        gettimeofday(&now,(void *)NULL);

	/* mark every second */
	mark_interval = USEC(&then,&now);
	if (mark_interval > 1000000){
	    {
		if(debug)printf("MARK--TIME: %d.%d --SEQ: %d --PREVMark: %d --PKTSIZE: %d --BITS %d\n",now.tv_sec -start.tv_sec,now.tv_usec - start.tv_usec,seq_no,mark_no,n,(seq_no - mark_no)*n*8); 
	        mark_no = seq_no;
            }
            mon->mark(now.tv_sec,now.tv_usec);
	    then.tv_sec = now.tv_sec;
	    then.tv_usec = now.tv_usec;
        }
    }
    exit(0);
}
